#include <SoftwareSerial.h>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <RTClib.h>

#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>

#define SERIAL MySerial
#define T_AN 9
#define T_AUS 21
#define BME280_KORR 1.0

//#define BME_SCK 13
//#define BME_MISO 12
//#define BME_MOSI 11
#define BME_CS 9 //SPI Chipselect für BME280

const int T = 200;

//Adafruit_BME280 bme; // I2C Druck, Temperatur, Feuchte
Adafruit_BME280 bme(BME_CS); //SPI Druck, Temperatur, Feuchte
RTC_DS1307 ds1307; //I2C RTC
DateTime time;
SoftwareSerial MySerial(7, 6);
File myFile;
int i, j, testByte;
int intervall = 60; //Minuten Messung
char c;
char StrTemp[10], StrDruck[10], StrFeuchte[10], StrStunde[5], StrMinute[5], StrTag[5], StrMonat[5], StrJahr[6];
char StrLine[40], StrIntervall[4];
char StrSpace[] = " ";
boolean bOk;

void testBytes() {
  myFile = SD.open("wd.txt");
  testByte = 0;
  if (myFile) {
    while (myFile.available()) {
      myFile.read();
      testByte++;
    }
    myFile.close();
  }
}

void getLine(char* StrLine) {
  StrLine[0] = 0;
  time = ds1307.now();
  itoa(time.day(), StrTag, 10); strcat(StrTag, StrSpace);
  itoa(time.month(), StrMonat, 10); strcat(StrMonat, StrSpace);
  ltoa(time.year(), StrJahr, 10); strcat(StrJahr, StrSpace);
  itoa(time.hour(), StrStunde, 10); strcat(StrStunde, StrSpace);
  itoa(time.minute(), StrMinute, 10); strcat(StrMinute, StrSpace);
  dtostrf(bme.readTemperature() - BME280_KORR, 6, 1, StrTemp); strcat(StrTemp, StrSpace);
  dtostrf(bme.readPressure() / 100.0, 6, 1, StrDruck); strcat(StrDruck, StrSpace);
  dtostrf(bme.readHumidity(), 6, 1, StrFeuchte);
  strcat(StrLine, StrTag);
  strcat(StrLine, StrMonat);
  strcat(StrLine, StrJahr);
  strcat(StrLine, StrStunde);
  strcat(StrLine, StrMinute);
  strcat(StrLine, StrTemp);
  strcat(StrLine, StrDruck);
  strcat(StrLine, StrFeuchte);
}

void setup() {
  Wire.begin();
  SERIAL.begin(9600);
  ds1307.begin();
  //ds1307.adjust(DateTime(F(__DATE__), F(__TIME__))); //Erste ds1307 Zeiteinstellung
  pinMode(10, OUTPUT); //Pin 10 SPI Chipselect für SD-Cardreader
  pinMode(9, OUTPUT); //Pin 9 SPI Chipselect für BME280
  pinMode(8, OUTPUT);  digitalWrite(8, HIGH); //Pin 8 Bluetoothstromversorgung
  bOk = true;
  if (!bme.begin()) { 
    Serial.println("BME280 error");
    return;
  }
  if (!SD.begin(10)) {                 //CS PIN 10
    Serial.println("SD Card error!");
    return;
  }
}

void loop() {
  time = ds1307.now();
  if (fmod(time.minute(), 60) == 0){            //Bluetooth an-/abschalten
    if(time.hour() == T_AUS) digitalWrite(8, LOW);
    if(time.hour() == T_AN) digitalWrite(8, HIGH); 
  }
  if (fmod(time.minute(), intervall) == 0 && bOk) {  //Messung auf SD speichern
    getLine(StrLine);
    myFile = SD.open("wd.txt", FILE_WRITE);
    if (myFile) {
      myFile.println(StrLine);
      myFile.close();
    }
    bOk = false;
  }
  if (fmod(time.minute(), intervall) != 0) bOk = true;
  if (SERIAL.available()) {
    c = SERIAL.read();
    if (c == 'A') {              // Lines senden
      //testBytes();
      //SERIAL.println(testByte); delay(T);
      myFile = SD.open("wd.txt");
      if (myFile) {
        while (myFile.available()) {
          SERIAL.write(myFile.read());
          //delay(T);
        }
        myFile.close();
      }
    }
    else if (c == 'B') {          // RTC Uhrzeit setzen
      j = 0;
      while (!SERIAL.available());
      while (SERIAL.available()) {
        StrLine[j] = SERIAL.read();
        j++;
        delay(T);
      }
      StrLine[j + 1] = 0;
      //SERIAL.println(StrLine);
      StrStunde[0] = 0; StrMinute[0] = 0; StrTag[0] = 0; StrMonat[0] = 0; StrJahr[0] = 0;
      strncpy(StrTag, StrLine, 2); StrTag[2] = 0;
      strncpy(StrMonat, StrLine + 2, 2); StrMonat[2] = 0;
      strncpy(StrJahr, StrLine + 4, 2); StrJahr[2] = 0;
      strncpy(StrStunde, StrLine + 6, 2); StrStunde[2] = 0;
      strncpy(StrMinute, StrLine + 8, 2); StrMinute[2] = 0;
      //ds1307.adjust(DateTime(atoi(StrJahr) + 2000 - 10, atoi(StrMonat) - 10, atoi(StrTag) - 10, atoi(StrStunde) - 10, atoi(StrMinute) - 10, 0));
      ds1307.adjust(DateTime(atoi(StrJahr) + 2000, atoi(StrMonat), atoi(StrTag), atoi(StrStunde), atoi(StrMinute), 0));
      StrStunde[0] = 0; StrMinute[0] = 0; StrTag[0] = 0; StrMonat[0] = 0; StrJahr[0] = 0; StrLine[0] = 0;
    }
    else if (c == 'C') {         // RTC Uhrzeit abfragen
      time = ds1307.now();
      SERIAL.print(time.day()); SERIAL.print(".");
      SERIAL.print(time.month()); SERIAL.print(".");
      SERIAL.print(time.year()); SERIAL.print(" ");
      SERIAL.print(time.hour()); SERIAL.print(":");
      SERIAL.println(time.minute());
    }
    else if (c == 'D') {           //WD.TXT löschen
      SD.remove("wd.txt");
    }
    else if (c == 'E') {           //Neues Messintervall setzten
      SERIAL.flush();
      StrIntervall[0] = 0;
      j = 0;
      while (!SERIAL.available());
      while (SERIAL.available()) {
        StrIntervall[j] = SERIAL.read();
        j++;
        delay(T);
      }
      intervall = atoi(StrIntervall);
      //SERIAL.println(intervall);
    }
    else if (c == 'F') {          //Aktuelle Daten ausgeben
      getLine(StrLine);
      SERIAL.println(StrLine);
    }
  }
}
